// This software use an external switch to select RF input Low or High.
// The switch should be connected between GND and to D5 of the Arduino.
// When switch is off, D5 will be pulled high by internal resistor.
// RF High input will be selected
// When switch is on, D5 will go low and  RF Low will be selected.
// Display will handle a printing and add dot to the measurement eg 2.500.000.000 Hz

#include <SPI.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>


#define PCF8574_I2C_Address 0x27                                      // Define the I2C address of Display

// ==============================================================================================================
//                                              Define Pins
// ==============================================================================================================

// Pins below are used by the Counter PCB and can not be changed
#define Ready 3                                                       // Define Ready Pin
#define Trigger 2                                                     // Define Trig Pin
#define RFInput 4                                                     // Define RF input Pin
#define SelectInput 5                                                 // Define RF input Switch
// ==============================================================================================================
//                                        Define LCD display 2x16 Char
// ==============================================================================================================

LiquidCrystal_I2C lcd(PCF8574_I2C_Address, 16, 2);                    // LCD address 

// ==============================================================================================================
//                                           Define Variables
// ==============================================================================================================

long Incomming_Data=0;                                                // Counter Value

// ==============================================================================================================
//                                              Functions
// ==============================================================================================================
//
// GetCounterValue ---------------------------------------------------------------------------------------- START

// Function:  GetCounterValue()
//            The function  "GetCounterValue" count the frequency from selected input.
//            The RF input is choosen by passing 0 or 1 to the function.  0=Low-Input, and 1=High-Input
//            Exmaple : GetCounterValue(0) will return the counting of Low input
//            Exmaple : GetCounterValue(1) will return the counting of High input

// Purpose:   Get Counter Value 
// Variables: n/a
// Return:    Counter Value
 
    long GetCounterValue(byte SignalInput){
    
	  unsigned long startedWaiting=0;                                   // Used for timeout
	  long CounterValue=0;                                              // Value from Counter

    // Check which input to connect to the counter (50 MHz input or 2.5 GHz)		
	  if(SignalInput==0) {
        digitalWrite(RFInput,LOW);                                    // Set Pin Low to select 50MHz input
	  }
        else{
        digitalWrite(RFInput,HIGH);                                   // Set Pin Low to select 2.5GHz input
	  }
    delay(50);
    // Send a trig pulse High-Low to start counting
    digitalWrite(Trigger,HIGH);                                       // Trigger goes High
    delayMicroseconds(100);
    digitalWrite(Trigger,LOW);                                        // Trigger goes Low
    
    delayMicroseconds(100);

    // Wait for the counter to reply
    // If no reply comes within 1.5 sec something is wrong and jump out 
    startedWaiting = millis();
    while(digitalRead(Ready)==0 && millis()-startedWaiting<=1500){
    }	
	  // Counter has replied and Counting value can be transfered to arduino
	  // Read all the bits to form the Counter value
    for (int Bitcounter = 1; Bitcounter <= 32; Bitcounter++) {
        digitalWrite(Trigger,HIGH);                                   // Trigger (Clock) goes High	
        delayMicroseconds(100);
		    CounterValue=CounterValue<<1;                                 // Shift Counter value one bit left
        if(digitalRead(Ready)==1){
            bitSet(CounterValue,0);                                   // If pin is high, set bit High 
        }
        digitalWrite(Trigger,LOW);                                    // Trigger (Clock) goes Low	
        delayMicroseconds(100);
    }  
    return CounterValue;                                              // Send back the counter Value
}	
// ---------------------------------------------------------------------------------------------------------- END
// PrintFormatLCD ----------------------------------------------------------------------------------------- START

// Function:  PrintFormatLCD()
//            The function  will format the countere value and print it on the display
// Purpose:   Print valule to display 
// Variables: n/a
// Return:    no return
    char LCDBuffer[20];
    
void PrintFormatLCD(long CounterValueLCD, byte InputChoise){
     
    lcd.setCursor(0, 1);                                              // Position Second row    
    sprintf(LCDBuffer,"%8ld",CounterValueLCD);                       // Convert Value to Char without padding zero
    
    // Check which Input is used and prirnt the result to the display
    if(InputChoise==1){
        // High input has been choosen so format text to present value
        lcd.print(LCDBuffer[0]); 
                   if(CounterValueLCD > 9999999) {
            lcd.print(".");                                           // Print dot
        } else {
            lcd.print(" ");
        }
                                     // Print Char  
        lcd.print(LCDBuffer[1]);                                      // Print Char
        lcd.print(LCDBuffer[2]);                                      // Print Char 
        lcd.print(LCDBuffer[3]);                                      // Print Char
        lcd.print(".");                                               // Print dot 
        lcd.print(LCDBuffer[4]);                                      // Print Char
        lcd.print(LCDBuffer[5]);                                      // Print Char 
        lcd.print(LCDBuffer[6]);                                      // Print Char  
        lcd.print(".");                                               // Print dot
        lcd.print(LCDBuffer[7]);                                      // Print Char
        lcd.print("0");                                               // Print "0"  
        lcd.print("0");                                               // Print "0"  
        lcd.print(" ");                                               // Print " " 
        lcd.print("H");                                               // Print "H" 
        lcd.print("z");                                               // Print "z"                                                
    }
    else{
        //Low input has been choosen so format text to present value
        lcd.print(LCDBuffer[0]);                                      // Print Char  
        lcd.print(LCDBuffer[1]); 
             if(CounterValueLCD > 999999) {
            lcd.print(".");                                           // Print dot
        } else {
            lcd.print(" ");
        }
                                     // Print Char                                      // Print Char
        lcd.print(LCDBuffer[2]);                                      // Print Char 
        lcd.print(LCDBuffer[3]);                                      // Print Char
        lcd.print(LCDBuffer[4]); 
              if(CounterValueLCD > 999) {
            lcd.print(".");                                           // Print dot
        } else {
            lcd.print(" ");
        }
                                     // Print Char                                     // Print Char        
        lcd.print(LCDBuffer[5]);                                      // Print Char 
        lcd.print(LCDBuffer[6]);                                      // Print Char  
        lcd.print(LCDBuffer[7]);                                      // Print Char
        lcd.print(" ");                                               // Print " " 
        lcd.print("H");                                               // Print "H" 
        lcd.print("z");                                               // Print "z"
        lcd.print(" ");                                               // Print " " 
        lcd.print(" ");                                               // Print " " 
        lcd.print(" ");                                               // Print " "                                                                         
     }
  }
// PrintFormatLCD ------------------------------------------------------------------------------------------- END

// ==============================================================================================================
//                                              SETUP
// ==============================================================================================================

void setup(){
    Serial.begin(9600);                                               // Init serial
    Wire.begin();                                                     // Init I2c

    pinMode(Trigger, OUTPUT);                                         // Set Pin as Output
    digitalWrite(Trigger,LOW);                                        // Set Pin Low     
    pinMode(RFInput, OUTPUT);                                         // Set Pin as Output 
    digitalWrite(RFInput,LOW);                                        // Set Pin Low
    pinMode(Ready, INPUT);                                            // Set pin as Input
    
    pinMode(SelectInput, INPUT_PULLUP);                               // Set pin as Input with interna pullup resistor	
	
    delay(500);                                                       // Shield to wake up

// LCD Initiering ----------------------------------------------------------------------------------------- START
    lcd.backlight();
    //lcd.noBacklight();

    lcd.init();                                                       // initialize the lcd 
    lcd.clear();                                                      // Clear display   
    lcd.setCursor(0, 0);                                              // Position  
    lcd.print("Frequency Input");                                     // Print Value  
// LCD Initiering ------------------------------------------------------------------------------------------- END

}



// ==============================================================================================================
//                                              Loop
// ==============================================================================================================
void loop() {

    if (digitalRead(SelectInput)==0){
        // Switch is low so count Low frequency			
        Incomming_Data=GetCounterValue(0);                            // Get Counter Value Low input
        PrintFormatLCD(Incomming_Data,0);                             // Print to Display        
    }
    else{
       // Switch is high so count high frequency			
        Incomming_Data=GetCounterValue(1);                            // Get Counter Value High input
        PrintFormatLCD(Incomming_Data,1);                             // Print to Display
    }
  
    //Serial.println(Incomming_Data,DEC);                               // Print out counter value
    
    }    
